# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Tensorflow set operations.
"""

import sys as _sys

from tensorflow.python.ops.sets_impl import set_difference
from tensorflow.python.ops.sets_impl import set_difference as difference
from tensorflow.python.ops.sets_impl import set_intersection
from tensorflow.python.ops.sets_impl import set_intersection as intersection
from tensorflow.python.ops.sets_impl import set_size
from tensorflow.python.ops.sets_impl import set_size as size
from tensorflow.python.ops.sets_impl import set_union
from tensorflow.python.ops.sets_impl import set_union as union